;------------------------------------------------------------------------------------------------
;	Console Prog to examine HIDs								;
;       Roger Stone April 2009									;
;												;
;Assemblewith:-	GoAsm USBtest.asm								;
;Link with:-	GoLink /console Kernel32.dll hid.dll setupapi.dll USBtest.obj -DEBUG COFF	;
;------------------------------------------------------------------------------------------------

#include \prog\gowindows.inc
#include \prog\all_api.inc

;------------------------------------------------------------------------------------------------
DATA SECTION
Startstrg db 01bh,"Looking for HID Devices",13,10,13,10
Errorstrg db 00eh,"Error time!",13,10
Donestrg  db 00bh,13,10,"No more",13,10
foundstrg  db 00bh,13,10,"HID found"
wrongstrg  db 00ch," - not Valid",13,10
Endstrg   db 022h,13,10,10,"That's all Folks",13,10,"Press Return"
VIDstrg  db 007h,13,10," VID "
PIDstrg  db 007h,13,10," PID "
Verstrg  db 007h,13,10," Ver "

filestring	db "COM1",0

HIDGuid			GUID 
InfoClassGuid		GUID

SP_DEVICE_INTERFACE_DATA STRUCT
cbSize			DD	
InferfaceClassGuid	GUID
Flags			DD
Reserved		DD
ENDSTRUCT

SP_DEVICE_INTERFACE_DETAIL_DATA STRUCT
cbSize			DD 	
string			DD 07ch dup 0
ENDSTRUCT

HIDD_ATTRIBUTES STRUCT
Size			DD ;ULONG
VendorID		DW ;USHORT
ProductID		DW ;USHORT
VersionNumber		DW ;USHORT
ENDSTRUCT

DevIntData		SP_DEVICE_INTERFACE_DATA
devdetailbuff		SP_DEVICE_INTERFACE_DETAIL_DATA 
Attributes		HIDD_ATTRIBUTES
StringBuff		DD 080h dup 0
hDevInfoSet		DD ?
hStdOut			DD ?
numchars		DD ?
hHeap			DD ?
hHIDDevice		DD ?
index			DD ?
length			DD ?
HIDpntr			DD ?
detaildatapntr		DD ?
chout			DD ?
;------------------------------------------------------------------------------------------------
CODE SECTION
START:
		push STD_OUTPUT_HANDLE
		call GetStdHandle
		mov [hStdOut], eax
		mov eax, offset Startstrg
		call OutStrg 

		;--------------------<  Obtain the GUID for the HID class >----------------------
		mov eax, offset HIDGuid
		invoke HidD_GetHidGuid, eax	

		;--------------------< Get device information set about all  HID devices  >------
		mov eax, offset HIDGuid
		invoke SetupDiGetClassDevsA, eax, 0, 0, 012h 	; returns handle to device info set

		cmp eax, INVALID_HANDLE_VALUE			; zero if no result 
		jz >error
		mov [hDevInfoSet], eax				; save handle to info set

		;--------------------<Get info about one device in the set >---------------------
		mov D [index], 0				; this will index  each device in the Info Set
		mov eax, SIZEOF DevIntData	
		mov [DevIntData.cbSize], eax
		mov eax, offset HIDGuid
		mov [HIDpntr], eax

a2:		invoke SetupDiEnumDeviceInterfaces, [hDevInfoSet], 0, [HIDpntr], [index], offset DevIntData	
		or eax,eax					; zero when no more HIDs to find
		jz >>alldone

		mov eax, offset foundstrg
		call OutStrg 
		;--------------------< Get the size of the details >-----------------------------
		invoke SetupDiGetDeviceInterfaceDetailA, [hDevInfoSet], offset DevIntData, 0, 0, offset length, 0
								; returns fail but fills in required length

		;--------------------< Get the details >-----------------------------------------
		mov eax,05h					; seems to be the required size
		mov [devdetailbuff],eax	
	
		invoke SetupDiGetDeviceInterfaceDetailA, [hDevInfoSet], offset DevIntData, offset devdetailbuff, [length], offset length, 0
		or eax,eax					; returns 0 if fail
		jz >>a4						; print message nd look for next HID

		mov eax,  offset devdetailbuff.string

		;--------------------< Get a handle to the device >------------------------------
		invoke CreateFile, eax, GENERIC_READ or GENERIC_WRITE, NULL, NULL, OPEN_EXISTING, 0, NULL
		cmp eax, INVALID_HANDLE_VALUE			; zero if no result 
		jz >>a4
		mov [hHIDDevice], eax

		;--------------------< Get data from the device >--------------------------------
		invoke HidD_GetAttributes, [hHIDDevice], Offset Attributes

		mov eax, offset VIDstrg
		call OutStrg
		mov eax, [Attributes.VendorID]
		call Outword 
		call Space
		invoke HidD_GetManufacturerString, [hHIDDevice], Offset StringBuff,080h
		mov eax, offset StringBuff
		call OutStrg2

		mov eax, offset PIDstrg
		call OutStrg
		mov eax, [Attributes.ProductID]
		call Outword 
		call Space
		invoke HidD_GetProductString, [hHIDDevice], Offset StringBuff,080h
		mov eax, offset StringBuff
		call OutStrg2

		mov eax, offset Verstrg
		call OutStrg
		mov eax, [Attributes.VersionNumber]
		call Outword 
		call Space
		;--------------------< Look for the next device >--------------------------------
a5:	
		inc d [index]					; index to next HID in set
		jmp long a2
		;----------
a4:					; print message and look for next HID
		mov eax, offset wrongstrg			; print message then look for next HID
		call OutStrg
		jmp <a5
		;----------
		;====================< Exits from Program >======================================
error:		mov eax, offset Errorstrg
		call OutStrg 
alldone:
		;--------------------< Free resources used by SetupDiGetClassDevs >--------------
		invoke SetupDiDestroyDeviceInfoList, [hDevInfoSet]	 	 
		invoke CloseHandle, [hHIDDevice]

		mov eax, offset Endstrg
		call OutStrg 

		push STD_INPUT_HANDLE
		call GetStdHandle
		push NULL
		push offset numchars
		push 1
		push offset StringBuff
		push eax
		call ReadFile

		invoke ExitProcess, 0
;================================================================================================
OutStrg2: 
s1:		movzx edx, w [eax]
		mov [chout], edx
		and edx, 00ffh
		jz >s2
		push eax
		call Outch
		pop eax
		inc eax
		inc eax			; 2 for wide chars
		jmp <s1
s2:		ret
;------------------------------------------------------------------------------------------------
OutStrg: 
 	push 0
	push offset numchars		; bytes written
	movzx edx,b [eax]
	inc eax
	push edx			; number of chars
	push eax			; adress of string
	push STD_OUTPUT_HANDLE
	call GetStdHandle
	mov [hStdOut], eax
	push eax
	call WriteFile
	ret
;-----------------------------------------------
Space:	
	mov eax, 020h
	mov [chout],eax
	jmp >Outch
;-----------------------------------------------
Outdword:
	push eax
	shr eax,10h
	call Outword
	pop eax
Outword:
	push eax
	shr eax, 8
	call bytout
	pop eax
bytout:	push eax
	shr eax,4 
	call nibout
	pop eax
nibout:	and eax,000Fh
	add al,90h
	daa
	adc al,40h
	daa
	mov [chout], eax
Outch:
 	push 0
	push offset numchars		; bytes written
	push 1				; number of chars
	push offset chout		; adress of string
	push [hStdOut]
	call WriteFile
	ret
;------------------------------------------------------------------------------------------------




